@@ -1,13 +1,13 @@
-use crate::{CallId, Recomposer, Slot, SlotId};
+use crate::{CallId, Slot, SlotId};
 use downcast_rs::{impl_downcast, Downcast};
-use log::trace;
-use std::{any::Any, collections::HashMap, panic::Location, pin::Pin};
+use log::{trace, warn};
+use std::{collections::HashMap, panic::Location};
 
-pub trait ComposeNode: Unpin + Downcast {}
+pub trait ComposeNode: Downcast {}
 impl_downcast!(ComposeNode);
-impl<T: Unpin + Downcast> ComposeNode for T {}
+impl<T: Downcast> ComposeNode for T {}
 
-type Tape = Vec<Slot<Pin<Box<dyn ComposeNode>>>>;
+type Tape = Vec<Slot<Box<dyn ComposeNode>>>;
 
 pub struct Composer {
     pub(crate) tape: Tape,
@@ -19,7 +19,18 @@ pub struct Composer {
 }
 
 impl Composer {
-    pub fn new(capacity: usize) -> Self {
+    pub fn new() -> Self {
+        Composer {
+            tape: Vec::new(),
+            slot_depth: Vec::new(),
+            depth: 0,
+            cursor: 0,
+            slot_key: None,
+            recycle_bin: HashMap::new(),
+        }
+    }
+
+    pub fn with_capacity(capacity: usize) -> Self {
         Composer {
             tape: Vec::with_capacity(capacity),
             slot_depth: Vec::with_capacity(capacity),
@@ -29,27 +40,6 @@ impl Composer {
             recycle_bin: HashMap::new(),
         }
     }
-}
-
-impl Composer {
-    pub fn finalize(mut self) -> Recomposer {
-        self.tape.truncate(self.cursor);
-        self.slot_depth.truncate(self.cursor);
-        self.cursor = 0;
-        self.depth = 0;
-        self.recycle_bin.clear();
-        Recomposer { composer: self }
-    }
-
-    pub fn finalize_with<F>(mut self, func: F) -> Recomposer
-    where
-        F: FnOnce(&mut Self),
-    {
-        self.cursor = 0;
-        self.depth = 0;
-        func(&mut self);
-        Recomposer { composer: self }
-    }
 
     #[track_caller]
     pub fn tag<F, T>(&mut self, key: usize, func: F) -> T
@@ -84,24 +74,14 @@ impl Composer {
         U: FnOnce(&mut Node),
         O: FnOnce(&Node) -> Output,
     {
-        self.slot(
-            factory,
-            false,
-            |_| {},
-            false,
-            |_, _| {},
-            skip,
-            update,
-            output,
-        )
+        self.group(factory, |_| {}, skip, update, output)
     }
 
     #[track_caller]
-    pub fn group_use_children<F, Node, C, S, A, U, O, Output>(
+    pub fn group<F, Node, C, S, U, O, Output>(
         &mut self,
         factory: F,
         children: C,
-        apply_children: A,
         skip: S,
         update: U,
         output: O,
@@ -111,16 +91,46 @@ impl Composer {
         Node: ComposeNode,
         C: FnOnce(&mut Composer),
         S: FnOnce(&mut Node) -> bool,
-        A: FnOnce(&mut Node, Vec<&dyn ComposeNode>),
         U: FnOnce(&mut Node),
         O: FnOnce(&Node) -> Output,
     {
         self.slot(
             factory,
-            true,
             children,
-            true,
+            |_, _| {},
+            false,
+            |_, _| {},
+            skip,
+            update,
+            output,
+        )
+    }
+
+    #[track_caller]
+    pub fn group_apply_children<F, Node, C, Children, AC, S, U, O, Output>(
+        &mut self,
+        factory: F,
+        children: C,
+        apply_children: AC,
+        skip: S,
+        update: U,
+        output: O,
+    ) -> Output
+    where
+        F: FnOnce(&mut Composer) -> Node,
+        Node: ComposeNode,
+        C: FnOnce(&mut Composer) -> Children,
+        AC: FnOnce(&mut Node, Children),
+        S: FnOnce(&mut Node) -> bool,
+        U: FnOnce(&mut Node),
+        O: FnOnce(&Node) -> Output,
+    {
+        self.slot(
+            factory,
+            children,
             apply_children,
+            false,
+            |_, _| {},
             skip,
             update,
             output,
@@ -128,10 +138,11 @@ impl Composer {
     }
 
     #[track_caller]
-    pub fn group<F, Node, C, S, A, U, O, Output>(
+    pub fn group_use_children<F, Node, C, UC, S, U, O, Output>(
         &mut self,
         factory: F,
         children: C,
+        use_children: UC,
         skip: S,
         update: U,
         output: O,
@@ -140,17 +151,17 @@ impl Composer {
         F: FnOnce(&mut Composer) -> Node,
         Node: ComposeNode,
         C: FnOnce(&mut Composer),
+        UC: FnOnce(&mut Node, Vec<&dyn ComposeNode>),
         S: FnOnce(&mut Node) -> bool,
-        A: FnOnce(&mut Node, Vec<&dyn ComposeNode>),
         U: FnOnce(&mut Node),
         O: FnOnce(&Node) -> Output,
     {
         self.slot(
             factory,
-            true,
             children,
-            false,
             |_, _| {},
+            true,
+            use_children,
             skip,
             update,
             output,
@@ -159,13 +170,13 @@ impl Composer {
 
     #[track_caller]
     #[allow(clippy::too_many_arguments)]
-    pub fn slot<F, Node, C, S, A, U, O, Output>(
+    pub fn slot<F, Node, C, Children, AC, UC, S, U, O, Output>(
         &mut self,
         factory: F,
-        has_children: bool,
         children: C,
-        use_children: bool,
-        apply_children: A,
+        apply_children: AC,
+        require_use_children: bool,
+        use_children: UC,
         skip: S,
         update: U,
         output: O,
@@ -173,9 +184,10 @@ impl Composer {
     where
         F: FnOnce(&mut Composer) -> Node,
         Node: ComposeNode,
-        C: FnOnce(&mut Composer),
+        C: FnOnce(&mut Composer) -> Children,
+        AC: FnOnce(&mut Node, Children),
+        UC: FnOnce(&mut Node, Vec<&dyn ComposeNode>),
         S: FnOnce(&mut Node) -> bool,
-        A: FnOnce(&mut Node, Vec<&dyn ComposeNode>),
         U: FnOnce(&mut Node),
         O: FnOnce(&Node) -> Output,
     {
@@ -209,8 +221,8 @@ impl Composer {
         let cached = self.tape.get_mut(cursor).map(|s| {
             // `Composer` required to guarantee `content` function not able to access current slot in self.tape
             // Otherwise need to wrap self.tape with RefCell to remove this unsafe but come with cost
-            let ptr = s.data.as_mut().expect("slot data").as_mut().get_mut();
-            let data = Box::leak(unsafe { Box::from_raw(ptr) });
+            let ptr = s.data.as_mut().expect("slot data").as_mut() as *mut dyn ComposeNode;
+            let data = unsafe { &mut *ptr };
             (s.id, s.size, data)
         });
 
@@ -228,12 +240,11 @@ impl Composer {
                         );
                         self.skip_slot(cursor, p_size);
                     } else {
-                        if has_children {
-                            children(self);
-                            if use_children {
-                                let c = self.children_of_slot_at(cursor);
-                                apply_children(node, c);
-                            }
+                        let c = children(self);
+                        apply_children(node, c);
+                        if require_use_children {
+                            let cn = self.children_of_slot_at(cursor);
+                            use_children(node, cn);
                         }
                         update(node);
                         self.end_slot_update(cursor);
@@ -241,9 +252,8 @@ impl Composer {
                     return output(node);
                 } else {
                     // NOTE:
-                    // same slot_id can only return same type as before under same root fn
-                    // However, this can happen when recompose with different root fn
-                    trace!(
+                    // same slot_id can only return same type as before under same root fn?
+                    warn!(
                         "{: >15} {} - {:?} - {:?}",
                         "downcast failed",
                         cursor,
@@ -259,18 +269,15 @@ impl Composer {
         self.begin_slot(cursor, slot_id);
         let mut node = factory(self);
 
-        if has_children {
-            children(self);
-
-            if use_children {
-                let c = self.children_of_slot_at(cursor);
-                apply_children(&mut node, c);
-            }
+        let c = children(self);
+        apply_children(&mut node, c);
+        if require_use_children {
+            let cn = self.children_of_slot_at(cursor);
+            use_children(&mut node, cn);
         }
 
         let out = output(&node);
-        // NOTE: expect node.into() will not change what node is
-        let data = Box::pin(node);
+        let data = Box::new(node);
         self.end_slot(cursor, data);
         out
     }
@@ -291,7 +298,7 @@ impl Composer {
             .filter_map(|c| {
                 self.tape
                     .get(c)
-                    .map(|s| s.data.as_ref().expect("slot data").as_ref().get_ref())
+                    .map(|s| s.data.as_ref().expect("slot data").as_ref())
             })
             .collect();
         children
@@ -303,7 +310,7 @@ impl Composer {
         trace!("{: >15} {} - {:?}", "begin_slot", cursor, slot_id);
     }
 
-    fn end_slot(&mut self, cursor: usize, data: Pin<Box<dyn ComposeNode>>) {
+    fn end_slot(&mut self, cursor: usize, data: Box<dyn ComposeNode>) {
         self.depth -= 1;
         let curr_cursor = self.current_cursor();
         if let Some(slot) = self.tape.get_mut(cursor) {
@@ -344,3 +351,9 @@ impl Composer {
         self.recycle_bin.insert(slot_id, slots);
     }
 }
+
+impl Default for Composer {
+    fn default() -> Self {
+        Self::new()
+    }
+}
